/*
 * Synopsis:
 *	char *disrcs(int stream, size_t *nchars, int *retval)
 *
 *	Gets a Data-is-Strings character string from <stream> and converts it
 *	into a counted string, returns a pointer to it, and puts the character
 *	count into *<nchars>.  The character string in <stream> consists of an
 *	unsigned integer, followed by a number of characters determined by the
 *	unsigned integer.
 *
 *	The data returned has an NULL byte appended to the end in case the
 *	calling program wishes to treat it as a NULL terminated string.
 *	This means the space allocated for the data is one byte larger than
 *	indicated by the count.
 *
 *	*<retval> gets DIS_SUCCESS if everything works well.  It gets an error
 *	code otherwise.  In case of an error, the <stream> character pointer is
 *	reset, making it possible to retry with some other conversion strategy.
 *	In case of an error, disrcs returns NULL and <nchars> is set to 0.
 */
#include <pbs_config.h>   /* the master config generated by configure */

#include <assert.h>
#include <stddef.h>
#include <stdlib.h>

#include "dis.h"
#include "dis_.h"

char *disrcs(stream, nchars, retval)
    int			stream;
    size_t		*nchars;
    int			*retval;
    {
	int		locret;
	int		negate;
	unsigned	count = 0;
	char		*value = NULL;

	assert(nchars != NULL);
	assert(retval != NULL);
	assert(dis_gets != NULL);
	assert(disr_commit != NULL);

	locret = disrsi_(stream, &negate, &count, 1);
	locret = negate ? DIS_BADSIGN : locret;
	if (locret == DIS_SUCCESS) {
		if (negate)
			locret = DIS_BADSIGN;
		else {
			value = (char *)malloc((size_t)count+1);
			if (value == NULL)
		        	locret = DIS_NOMALLOC;
			else {
				if ((*dis_gets)(stream, value,
						(size_t)count) != (size_t)count)
			        	locret = DIS_PROTO;
				else
					value[count] = '\0';
			}
		}
	}
	locret = ((*disr_commit)(stream, locret == DIS_SUCCESS) < 0) ?
	    DIS_NOCOMMIT : locret;
	if ((*retval = locret) != DIS_SUCCESS && value != NULL) {
		count = 0;
		free(value);
		value = NULL;
	}
	*nchars = count;
        return (value);
}
